@@ -48,7 +48,7 @@ class NoseGAE(Plugin):
         if options.gae_lib_root is not None:
             root = self._gae_path = options.gae_lib_root
             for d in self.lib_dirs:
-                sys.path.append(os.path.join(root, d))
+                sys.path.insert(0, os.path.join(root, d))
         else:
             self._gae_path = None
         try:
@@ -66,13 +66,6 @@ class NoseGAE(Plugin):
             import webob
             import yaml
             import django
-            self._preserve_mods = dict(
-                (name, sys.modules[name]) for name in sys.modules.keys()
-                if (name.startswith('google')
-                    or name.startswith('webob')
-                    or name.startswith('yaml')
-                    or name.startswith('django')
-                    or name.startswith('nose')))
         except ImportError, e:
             self.enabled = False
             warn("Google App Engine not found in %s" % options.gae_lib_root,
@@ -110,7 +103,7 @@ class NoseGAE(Plugin):
             dev_appserver = self._gae['dev_appserver']
             sandbox_root = self._path
             testMatch = self.config.testMatch
-            preserve = self._preserve_mods.copy()
+            module_dict = self._setup_shared_modules()
         self.hook = Hook(sys.modules)
         sys.meta_path = [self.hook]
         # set up allowed file access paths
@@ -118,7 +111,13 @@ class NoseGAE(Plugin):
         if self._gae_path:
             paths.append(self._gae_path)
         dev_appserver.FakeFile.SetAllowedPaths(paths)
-                                                
+
+    def _setup_shared_modules(self):
+        mods = self._gae['dev_appserver'].SetupSharedModules(sys.modules)
+        mods.update(dict((name, mod) for name, mod in sys.modules.items()
+                         if name.startswith('nose')))
+        return mods
+        
         
 class HookMixin(object):
     """
@@ -153,6 +152,9 @@ class HookMixin(object):
     def load_module(self, fullname):
         # only called when sandboxed
         try:
+            # FIXME: possible strategy for sandboxing file, open, etc
+            # if mod.file is <type 'file'> or nothing, set it to
+            # FakeFile. Same for mod.open.
             return super(HookMixin, self).load_module(fullname)
         finally:
             if fullname == self.sandbox:
@@ -165,7 +167,8 @@ class HookMixin(object):
         self.sandbox = mod_name
         self._old_modules = sys.modules.copy()
         self.dev_appserver.ClearAllButEncodingsModules(sys.modules)
-        sys.modules.update(self.preserve)
+        # restore shared modules (see issue #2)
+        sys.modules.update(self.module_dict)
         if hasattr(sys, 'path_importer_cache'):
             sys.path_importer_cache.clear()
     
@@ -177,6 +180,8 @@ class HookMixin(object):
             return
         self.log("<<< EXIT sandbox %s" % self.sandbox)
         self.sandbox = None
+        # preserve loaded modules for next entry into sandbox (see issue #7)
+        self.module_dict.update(sys.modules)
         sys.modules.update(self._old_modules)
         if hasattr(sys, 'path_importer_cache'):
             sys.path_importer_cache.clear()
